/*
 * Copyright (c) 1988-1997 Sam Leffler
 * Copyright (c) 1991-1997 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Sam Leffler and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Sam Leffler and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL SAM LEFFLER OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#ifndef _TIFF_
#define	_TIFF_

#include "tiffconf.h"

#include <stdint.h>
#include "tifflong.h"
#if !LL_HAVE_UINT64
#undef TIFF_UINT64_T
#undef TIFF_INT64_T
#define TIFF_UINT64_T ll_uint64_t
#define TIFF_INT64_T ll_int64_t
#endif
#if defined(__MSHORT__) || defined(__PUREC__) || defined(__AHCC__)
#undef TIFF_UINT32_T
#undef TIFF_INT32_T
#define TIFF_UINT32_T unsigned long
#define TIFF_INT32_T signed long
#endif

#if !defined(__gcc_double_t_defined)
struct __gcc_double {
	unsigned long hi, lo;
};
#if defined(TIFF_SLB) && !defined(__GNUC__)
#define __gcc_double_t struct __gcc_double
#else
#define __gcc_double_t double
#endif
#define __gcc_double_t_defined
#endif
#if !defined(__gcc_float_t_defined)
#if defined(TIFF_SLB) && !defined(__GNUC__)
#define __gcc_float_t unsigned long
#else
#define __gcc_float_t float
#endif
#define __gcc_float_t_defined
#endif

#if (defined(__MSHORT__) || defined(__PUREC__) || defined(__AHCC__)) && defined(TIFF_SLB) && (defined(__atarist__) || defined(__TOS__))
typedef long tiff_int_t;
typedef unsigned long tiff_uint_t;
typedef tiff_uint_t tiff_uint16_t;
#define tiff_enum(x) tiff_int_t
#else
typedef int tiff_int_t;
typedef unsigned int tiff_uint_t;
typedef TIFF_UINT16_T tiff_uint16_t;
#define tiff_enum(x) x
#endif
#if defined(TIFF_SLB) && (defined(__atarist__) || defined(__TOS__))
#  include <features.h>
#  define TIFF_CALLBACK __CDECL
#  define TIFF_API __CDECL
#else
#  define TIFF_API
#  define TIFF_CALLBACK
#endif

/*
 * Tag Image File Format (TIFF)
 *
 * Based on Rev 6.0 from:
 *    Developer's Desk
 *    Aldus Corporation
 *    411 First Ave. South
 *    Suite 200
 *    Seattle, WA  98104
 *    206-622-5500
 *
 *    (http://partners.adobe.com/asn/developer/PDFS/TN/TIFF6.pdf)
 *
 * For BigTIFF design notes see the following links
 *    http://www.remotesensing.org/libtiff/bigtiffdesign.html
 *    http://www.awaresystems.be/imaging/tiff/bigtiff.html
 */

#define TIFF_VERSION_CLASSIC 42
#define TIFF_VERSION_BIG 43

#define TIFF_BIGENDIAN      0x4d4d
#define TIFF_LITTLEENDIAN   0x4949
#define MDI_LITTLEENDIAN    0x5045
#define MDI_BIGENDIAN       0x4550

/*
 * Intrinsic data types required by the file format:
 *
 * 8-bit quantities     int8/uint8
 * 16-bit quantities    int16/uint16
 * 32-bit quantities    int32/uint32
 * 64-bit quantities    int64/uint64
 * strings              unsigned char*
 */

typedef TIFF_INT8_T   int8;
typedef TIFF_UINT8_T  uint8;

typedef TIFF_INT16_T  int16;
typedef TIFF_UINT16_T uint16;

typedef TIFF_INT32_T  int32;
typedef TIFF_UINT32_T uint32;

typedef TIFF_INT64_T  int64;
typedef TIFF_UINT64_T uint64;

/*
 * Some types as promoted in a variable argument list
 * We use uint16_vap rather then directly using int, because this way
 * we document the type we actually want to pass through, conceptually,
 * rather then confusing the issue by merely stating the type it gets
 * promoted to
 */

typedef tiff_uint_t uint16_vap;

/*
 * TIFF header.
 */
typedef struct {
	uint16 tiff_magic;      /* magic number (defines byte order) */
	uint16 tiff_version;    /* TIFF version number */
} TIFFHeaderCommon;
typedef struct {
	uint16 tiff_magic;      /* magic number (defines byte order) */
	uint16 tiff_version;    /* TIFF version number */
	uint32 tiff_diroff;     /* byte offset to first directory */
} TIFFHeaderClassic;
typedef struct {
	uint16 tiff_magic;      /* magic number (defines byte order) */
	uint16 tiff_version;    /* TIFF version number */
	uint16 tiff_offsetsize; /* size of offsets, should be 8 */
	uint16 tiff_unused;     /* unused word, should be 0 */
	uint64 tiff_diroff;     /* byte offset to first directory */
} TIFFHeaderBig;


/*
 * NB: In the comments below,
 *  - items marked with a + are obsoleted by revision 5.0,
 *  - items marked with a ! are introduced in revision 6.0.
 *  - items marked with a % are introduced post revision 6.0.
 *  - items marked with a $ are obsoleted by revision 6.0.
 *  - items marked with a & are introduced by Adobe DNG specification.
 */

/*
 * Tag data type information.
 *
 * Note: RATIONALs are the ratio of two 32-bit integer values.
 */
typedef enum {
	TIFF_NOTYPE = 0,      /* placeholder */
	TIFF_BYTE = 1,        /* 8-bit unsigned integer */
	TIFF_ASCII = 2,       /* 8-bit bytes w/ last byte null */
	TIFF_SHORT = 3,       /* 16-bit unsigned integer */
	TIFF_LONG = 4,        /* 32-bit unsigned integer */
	TIFF_RATIONAL = 5,    /* 64-bit unsigned fraction */
	TIFF_SBYTE = 6,       /* !8-bit signed integer */
	TIFF_UNDEFINED = 7,   /* !8-bit untyped data */
	TIFF_SSHORT = 8,      /* !16-bit signed integer */
	TIFF_SLONG = 9,       /* !32-bit signed integer */
	TIFF_SRATIONAL = 10,  /* !64-bit signed fraction */
	TIFF_FLOAT = 11,      /* !32-bit IEEE floating point */
	TIFF_DOUBLE = 12,     /* !64-bit IEEE floating point */
	TIFF_IFD = 13,        /* %32-bit unsigned integer (offset) */
	TIFF_LONG8 = 16,      /* BigTIFF 64-bit unsigned integer */
	TIFF_SLONG8 = 17,     /* BigTIFF 64-bit signed integer */
	TIFF_IFD8 = 18        /* BigTIFF 64-bit unsigned integer (offset) */
} TIFFDataType;

/*
 * TIFF Tag Definitions.
 */
#define	TIFFTAG_SUBFILETYPE				254	/* LONG subfile data descriptor */
#define	    FILETYPE_REDUCEDIMAGE			0x1	/* reduced resolution version */
#define	    FILETYPE_PAGE					0x2	/* one page of many */
#define	    FILETYPE_MASK					0x4	/* transparency mask */
#define	TIFFTAG_OSUBFILETYPE			255	/* SHORT +kind of data in subfile */
#define	    OFILETYPE_IMAGE					1	/* full resolution image data */
#define	    OFILETYPE_REDUCEDIMAGE			2	/* reduced size image data */
#define	    OFILETYPE_PAGE					3	/* one page of many */
#define	TIFFTAG_IMAGEWIDTH				256	/* LONG image width in pixels */
#define	TIFFTAG_IMAGELENGTH				257	/* LONG image height in pixels */
#define	TIFFTAG_BITSPERSAMPLE			258	/* SHORT bits per channel (sample) */
#define	TIFFTAG_COMPRESSION				259	/* SHORT data compression technique */
#define	    COMPRESSION_NONE				1	/* dump mode */
#define	    COMPRESSION_CCITTRLE			2	/* CCITT modified Huffman RLE */
#define	    COMPRESSION_CCITTFAX3			3	/* CCITT Group 3 fax encoding */
#define     COMPRESSION_CCITT_T4    		3   /* CCITT T.4 (TIFF 6 name) */
#define	    COMPRESSION_CCITTFAX4			4	/* CCITT Group 4 fax encoding */
#define     COMPRESSION_CCITT_T6    		4   /* CCITT T.6 (TIFF 6 name) */
#define	    COMPRESSION_LZW					5   /* Lempel-Ziv  & Welch */
#define	    COMPRESSION_OJPEG				6	/* !6.0 JPEG */
#define	    COMPRESSION_JPEG				7	/* %JPEG DCT compression */
#define     COMPRESSION_T85					9	/* !TIFF/FX T.85 JBIG compression */
#define     COMPRESSION_T43					10	/* !TIFF/FX T.43 colour by layered JBIG compression */
#define	    COMPRESSION_NEXT				UINT32_C(32766)	/* NeXT 2-bit RLE */
#define	    COMPRESSION_CCITTRLEW			UINT32_C(32771)	/* #1 w/ word alignment */
#define	    COMPRESSION_PACKBITS			UINT32_C(32773)	/* Macintosh RLE */
#define	    COMPRESSION_THUNDERSCAN			UINT32_C(32809)	/* ThunderScan RLE */
/* codes 32895-32898 are reserved for ANSI IT8 TIFF/IT <dkelly@apago.com) */
#define	    COMPRESSION_IT8CTPAD			UINT32_C(32895)   /* IT8 CT w/padding */
#define	    COMPRESSION_IT8LW				UINT32_C(32896)   /* IT8 Linework RLE */
#define	    COMPRESSION_IT8MP				UINT32_C(32897)   /* IT8 Monochrome picture */
#define	    COMPRESSION_IT8BL				UINT32_C(32898)   /* IT8 Binary line art */
/* compression codes 32908-32911 are reserved for Pixar */
#define     COMPRESSION_PIXARFILM			UINT32_C(32908)   /* Pixar companded 10bit LZW */
#define	    COMPRESSION_PIXARLOG			UINT32_C(32909)   /* Pixar companded 11bit ZIP */
#define	    COMPRESSION_DEFLATE				UINT32_C(32946)	/* Deflate compression */
#define     COMPRESSION_ADOBE_DEFLATE   	8       /* Deflate compression,
						   as recognized by Adobe */
/* compression code 32947 is reserved for Oceana Matrix <dev@oceana.com> */
#define     COMPRESSION_DCS        	 		UINT32_C(32947)   /* Kodak DCS encoding */
#define	    COMPRESSION_JBIG				UINT32_C(34661)	/* ISO JBIG */
#define     COMPRESSION_SGILOG				UINT32_C(34676)	/* SGI Log Luminance RLE */
#define     COMPRESSION_SGILOG24			UINT32_C(34677)	/* SGI Log 24-bit packed */
#define     COMPRESSION_JP2000      		UINT32_C(34712)   /* Leadtools JPEG2000 */
#define     COMPRESSION_LERC        		UINT32_C(34887)   /* ESRI Lerc codec: https://github.com/Esri/lerc */
/* compression codes 34887-34889 are reserved for ESRI */
#define	    COMPRESSION_LZMA				UINT32_C(34925)	/* LZMA2 */
#define	    COMPRESSION_ZSTD				UINT32_C(50000)	/* ZSTD: WARNING not registered in Adobe-maintained registry */
#define	    COMPRESSION_WEBP				UINT32_C(50001)	/* WEBP: WARNING not registered in Adobe-maintained registry */
#define	TIFFTAG_PHOTOMETRIC				262	/* SHORT photometric interpretation */
#define	    PHOTOMETRIC_MINISWHITE			0	/* min value is white */
#define	    PHOTOMETRIC_MINISBLACK			1	/* min value is black */
#define	    PHOTOMETRIC_RGB					2	/* RGB color model */
#define	    PHOTOMETRIC_PALETTE				3	/* color map indexed */
#define	    PHOTOMETRIC_MASK				4	/* $holdout mask */
#define	    PHOTOMETRIC_SEPARATED			5	/* !color separations */
#define	    PHOTOMETRIC_YCBCR				6	/* !CCIR 601 */
#define	    PHOTOMETRIC_CIELAB				8	/* !1976 CIE L*a*b* */
#define	    PHOTOMETRIC_ICCLAB				9	/* ICC L*a*b* [Adobe TIFF Technote 4] */
#define	    PHOTOMETRIC_ITULAB				10	/* ITU L*a*b* */
#define	    PHOTOMETRIC_CFA					UINT32_C(32803)	/* color filter array */
#define     PHOTOMETRIC_LOGL				UINT32_C(32844)	/* CIE Log2(L) */
#define     PHOTOMETRIC_LOGLUV				UINT32_C(32845)	/* CIE Log2(L) (u',v') */
#define	TIFFTAG_THRESHHOLDING			263	/* SHORT +thresholding used on data */
#define	    THRESHHOLD_BILEVEL				1	/* b&w art scan */
#define	    THRESHHOLD_HALFTONE				2	/* or dithered scan */
#define	    THRESHHOLD_ERRORDIFFUSE			3	/* usually floyd-steinberg */
#define	TIFFTAG_CELLWIDTH				264	/* SHORT +dithering matrix width */
#define	TIFFTAG_CELLLENGTH				265	/* SHORT +dithering matrix height */
#define	TIFFTAG_FILLORDER				266	/* SHORT data order within a byte */
#define	    FILLORDER_MSB2LSB				1	/* most significant -> least */
#define	    FILLORDER_LSB2MSB				2	/* least significant -> most */
#define	TIFFTAG_DOCUMENTNAME			269	/* ASCII name of doc. image is from */
#define	TIFFTAG_IMAGEDESCRIPTION		270	/* ASCII info about image */
#define	TIFFTAG_MAKE					271	/* ASCII scanner manufacturer name */
#define	TIFFTAG_MODEL					272	/* ASCII scanner model name/number */
#define	TIFFTAG_STRIPOFFSETS			273	/* LONG8 offsets to data strips */
#define	TIFFTAG_ORIENTATION				274	/* SHORT +image orientation */
#define	    ORIENTATION_TOPLEFT				1	/* row 0 top, col 0 lhs */
#define	    ORIENTATION_TOPRIGHT			2	/* row 0 top, col 0 rhs */
#define	    ORIENTATION_BOTRIGHT			3	/* row 0 bottom, col 0 rhs */
#define	    ORIENTATION_BOTLEFT				4	/* row 0 bottom, col 0 lhs */
#define	    ORIENTATION_LEFTTOP				5	/* row 0 lhs, col 0 top */
#define	    ORIENTATION_RIGHTTOP			6	/* row 0 rhs, col 0 top */
#define	    ORIENTATION_RIGHTBOT			7	/* row 0 rhs, col 0 bottom */
#define	    ORIENTATION_LEFTBOT				8	/* row 0 lhs, col 0 bottom */
#define	TIFFTAG_SAMPLESPERPIXEL			277	/* SHORT samples per pixel */
#define	TIFFTAG_ROWSPERSTRIP			278	/* LONG rows per strip of data */
#define	TIFFTAG_STRIPBYTECOUNTS			279	/* LONG8 bytes counts for strips */
#define	TIFFTAG_MINSAMPLEVALUE			280	/* SHORT +minimum sample value */
#define	TIFFTAG_MAXSAMPLEVALUE			281	/* SHORT +maximum sample value */
#define	TIFFTAG_XRESOLUTION				282	/* RATIONAL pixels/resolution in x */
#define	TIFFTAG_YRESOLUTION				283	/* RATIONAL pixels/resolution in y */
#define	TIFFTAG_PLANARCONFIG			284	/* SHORT storage organization */
#define	    PLANARCONFIG_CONTIG				1	/* single image plane */
#define	    PLANARCONFIG_SEPARATE			2	/* separate planes of data */
#define	TIFFTAG_PAGENAME				285	/* ASCII page name image is from */
#define	TIFFTAG_XPOSITION				286	/* RATIONAL x page offset of image lhs */
#define	TIFFTAG_YPOSITION				287	/* RATIONAL y page offset of image lhs */
#define	TIFFTAG_FREEOFFSETS				288	/* LONG8 +byte offset to free block */
#define	TIFFTAG_FREEBYTECOUNTS			289	/* LONG8 +sizes of free blocks */
#define	TIFFTAG_GRAYRESPONSEUNIT		290	/* SHORT $gray scale curve accuracy */
#define	    GRAYRESPONSEUNIT_10S			1	/* tenths of a unit */
#define	    GRAYRESPONSEUNIT_100S			2	/* hundredths of a unit */
#define	    GRAYRESPONSEUNIT_1000S			3	/* thousandths of a unit */
#define	    GRAYRESPONSEUNIT_10000S			4	/* ten-thousandths of a unit */
#define	    GRAYRESPONSEUNIT_100000S		5	/* hundred-thousandths */
#define	TIFFTAG_GRAYRESPONSECURVE		291	/* SHORT $gray scale response curve */
#define	TIFFTAG_GROUP3OPTIONS			292	/* LONG 32 flag bits */
#define	TIFFTAG_T4OPTIONS				292	/* LONG TIFF 6.0 proper name alias */
#define	    GROUP3OPT_2DENCODING			0x1	/* 2-dimensional coding */
#define	    GROUP3OPT_UNCOMPRESSED			0x2	/* data not compressed */
#define	    GROUP3OPT_FILLBITS				0x4	/* fill to byte boundary */
#define	TIFFTAG_GROUP4OPTIONS			293	/* LONG 32 flag bits */
#define TIFFTAG_T6OPTIONS           	293 /* LONG TIFF 6.0 proper name */
#define	    GROUP4OPT_UNCOMPRESSED			0x2	/* data not compressed */
#define	TIFFTAG_RESOLUTIONUNIT			296	/* SHORT units of resolutions */
#define	    RESUNIT_NONE					1	/* no meaningful units */
#define	    RESUNIT_INCH					2	/* english */
#define	    RESUNIT_CENTIMETER				3	/* metric */
#define	TIFFTAG_PAGENUMBER				297	/* SHORT[2] page numbers of multi-page */
#define	TIFFTAG_COLORRESPONSEUNIT		300	/* SHORT $color curve accuracy */
#define	    COLORRESPONSEUNIT_10S			1	/* tenths of a unit */
#define	    COLORRESPONSEUNIT_100S			2	/* hundredths of a unit */
#define	    COLORRESPONSEUNIT_1000S			3	/* thousandths of a unit */
#define	    COLORRESPONSEUNIT_10000S		4	/* ten-thousandths of a unit */
#define	    COLORRESPONSEUNIT_100000S		5	/* hundred-thousandths */
#define	TIFFTAG_TRANSFERFUNCTION		301	/* SHORT[4] !colorimetry info */
#define	TIFFTAG_SOFTWARE				305	/* ASCII name & release */
#define	TIFFTAG_DATETIME				306	/* ASCII creation date and time */
#define	TIFFTAG_ARTIST					315	/* ASCII creator of image */
#define	TIFFTAG_HOSTCOMPUTER			316	/* ASCII machine where created */
#define	TIFFTAG_PREDICTOR				317	/* SHORT prediction scheme w/ LZW */
#define     PREDICTOR_NONE					1	/* no prediction scheme used */
#define     PREDICTOR_HORIZONTAL			2	/* horizontal differencing */
#define     PREDICTOR_FLOATINGPOINT			3	/* floating point predictor */
#define	TIFFTAG_WHITEPOINT				318	/* RATIONAL image white point */
#define	TIFFTAG_PRIMARYCHROMATICITIES	319	/* RATIONAL !primary chromaticities */
#define	TIFFTAG_COLORMAP				320	/* SHORT *[4] RGB map for palette image */
#define	TIFFTAG_HALFTONEHINTS			321	/* SHORT[2] !highlight+shadow info */
#define	TIFFTAG_TILEWIDTH				322	/* LONG !tile width in pixels */
#define	TIFFTAG_TILELENGTH				323	/* LONG !tile height in pixels */
#define TIFFTAG_TILEOFFSETS				324	/* LONG8 !offsets to data tiles */
#define TIFFTAG_TILEBYTECOUNTS			325	/* LONG8 !byte counts for tiles */
#define	TIFFTAG_BADFAXLINES				326	/* LONG lines w/ wrong pixel count */
#define	TIFFTAG_CLEANFAXDATA			327	/* SHORT regenerated line info */
#define	    CLEANFAXDATA_CLEAN				0	/* no errors detected */
#define	    CLEANFAXDATA_REGENERATED		1	/* receiver regenerated lines */
#define	    CLEANFAXDATA_UNCLEAN			2	/* uncorrected errors exist */
#define	TIFFTAG_CONSECUTIVEBADFAXLINES	328	/* LONG max consecutive bad lines */
#define	TIFFTAG_SUBIFD					330	/* IFD8 subimage descriptors */
#define	TIFFTAG_INKSET					332	/* SHORT !inks in separated image */
#define	    INKSET_CMYK						1	/* !cyan-magenta-yellow-black color */
#define	    INKSET_MULTIINK					2	/* !multi-ink or hi-fi color */
#define	TIFFTAG_INKNAMES				333	/* ASCII !ascii names of inks */
#define	TIFFTAG_NUMBEROFINKS			334	/* SHORT !number of inks */
#define	TIFFTAG_DOTRANGE				336	/* SHORT !0% and 100% dot codes */
#define	TIFFTAG_TARGETPRINTER			337	/* ASCII !separation target */
#define	TIFFTAG_EXTRASAMPLES			338	/* SHORT[]  !info about extra samples */
#define	    EXTRASAMPLE_UNSPECIFIED			0	/* !unspecified data */
#define	    EXTRASAMPLE_ASSOCALPHA			1	/* !associated alpha data */
#define	    EXTRASAMPLE_UNASSALPHA			2	/* !unassociated alpha data */
#define	TIFFTAG_SAMPLEFORMAT			339	/* SHORT !data sample format */
#define	    SAMPLEFORMAT_UINT				1	/* !unsigned integer data */
#define	    SAMPLEFORMAT_INT				2	/* !signed integer data */
#define	    SAMPLEFORMAT_IEEEFP				3	/* !IEEE floating point data */
#define	    SAMPLEFORMAT_VOID				4	/* !untyped data */
#define	    SAMPLEFORMAT_COMPLEXINT			5	/* !complex signed int */
#define	    SAMPLEFORMAT_COMPLEXIEEEFP		6	/* !complex ieee floating */
#define	TIFFTAG_SMINSAMPLEVALUE			340	/* ANY !variable MinSampleValue */
#define	TIFFTAG_SMAXSAMPLEVALUE			341	/* ANY !variable MaxSampleValue */
#define	TIFFTAG_CLIPPATH				343	/* BYTE %ClipPath [Adobe TIFF technote 2] */
#define	TIFFTAG_XCLIPPATHUNITS			344	/* SLONG %XClipPathUnits [Adobe TIFF technote 2] */
#define	TIFFTAG_YCLIPPATHUNITS			345	/* SLONG  %YClipPathUnits [Adobe TIFF technote 2] */
#define	TIFFTAG_INDEXED					346	/* SHORT %Indexed [Adobe TIFF Technote 3] */
#define	TIFFTAG_JPEGTABLES				347	/* UNDEFINED %JPEG table stream */
#define	TIFFTAG_OPIPROXY				351	/* %OPI Proxy [Adobe TIFF technote] */
/* Tags 400-435 are from the TIFF/FX spec */
#define TIFFTAG_GLOBALPARAMETERSIFD		400	/* IFD8 ! */
#define TIFFTAG_PROFILETYPE				401	/* LONG ! */
#define     PROFILETYPE_UNSPECIFIED			0	/* ! */
#define     PROFILETYPE_G3_FAX				1	/* ! */
#define TIFFTAG_FAXPROFILE				402	/* BYTE ! */
#define     FAXPROFILE_S					1	/* !TIFF/FX FAX profile S */
#define     FAXPROFILE_F					2	/* !TIFF/FX FAX profile F */
#define     FAXPROFILE_J					3	/* !TIFF/FX FAX profile J */
#define     FAXPROFILE_C					4	/* !TIFF/FX FAX profile C */
#define     FAXPROFILE_L					5	/* !TIFF/FX FAX profile L */
#define     FAXPROFILE_M					6	/* !TIFF/FX FAX profile LM */
#define TIFFTAG_CODINGMETHODS			403	/* LONG !TIFF/FX coding methods */
#define     CODINGMETHODS_T4_1D				(1 << 1)	/* !T.4 1D */
#define     CODINGMETHODS_T4_2D				(1 << 2)	/* !T.4 2D */
#define     CODINGMETHODS_T6				(1 << 3)	/* !T.6 */
#define     CODINGMETHODS_T85 				(1 << 4)	/* !T.85 JBIG */
#define     CODINGMETHODS_T42 				(1 << 5)	/* !T.42 JPEG */
#define     CODINGMETHODS_T43				(1 << 6)	/* !T.43 colour by layered JBIG */
#define TIFFTAG_VERSIONYEAR				404	/* BYTE !TIFF/FX version year */
#define TIFFTAG_MODENUMBER				405	/* BYTE !TIFF/FX mode number */
#define TIFFTAG_DECODE					433	/* SRATIONAL !TIFF/FX decode */
#define TIFFTAG_IMAGEBASECOLOR			434	/* SHORT !TIFF/FX image base colour */
#define TIFFTAG_T82OPTIONS				435	/* LONG !TIFF/FX T.82 options */
/*
 * Tags 512-521 are obsoleted by Technical Note #2 which specifies a
 * revised JPEG-in-TIFF scheme.
 */
#define	TIFFTAG_JPEGPROC				512	/* SHORT !JPEG processing algorithm */
#define	    JPEGPROC_BASELINE				1	/* !baseline sequential */
#define	    JPEGPROC_LOSSLESS				14	/* !Huffman coded lossless */
#define	TIFFTAG_JPEGIFOFFSET			513	/* LONG8 !pointer to SOI marker */
#define	TIFFTAG_JPEGIFBYTECOUNT			514	/* LONG8 !JFIF stream length */
#define	TIFFTAG_JPEGRESTARTINTERVAL		515	/* SHORT !restart interval length */
#define	TIFFTAG_JPEGLOSSLESSPREDICTORS	517	/* !lossless proc predictor */
#define	TIFFTAG_JPEGPOINTTRANSFORM		518	/* !lossless point transform */
#define	TIFFTAG_JPEGQTABLES				519	/* VARIABLE2 !Q matrix offsets */
#define	TIFFTAG_JPEGDCTABLES			520	/* VARIABLE2 !DCT table offsets */
#define	TIFFTAG_JPEGACTABLES			521	/* VARIABLE2 !AC coefficient offsets */
#define	TIFFTAG_YCBCRCOEFFICIENTS		529	/* RATIONAL !RGB -> YCbCr transform */
#define	TIFFTAG_YCBCRSUBSAMPLING		530	/* SHORT !YCbCr subsampling factors */
#define	TIFFTAG_YCBCRPOSITIONING		531	/* SHORT !subsample positioning */
#define	    YCBCRPOSITION_CENTERED			1	/* !as in PostScript Level 2 */
#define	    YCBCRPOSITION_COSITED			2	/* !as in CCIR 601-1 */
#define	TIFFTAG_REFERENCEBLACKWHITE		532	/* RATIONAL !colorimetry info */
#define TIFFTAG_STRIPROWCOUNTS			559 /* LONG !TIFF/FX strip row counts */
#define	TIFFTAG_XMLPACKET				700	/* BYTE %XML packet [Adobe XMP Specification, January 2004 */
#define TIFFTAG_OPIIMAGEID				UINT32_C(32781)	/* %OPI ImageID [Adobe TIFF technote] */
/* tags 32952-32956 are private tags registered to Island Graphics */
#define TIFFTAG_REFPTS					UINT32_C(32953)	/* image reference points */
#define TIFFTAG_REGIONTACKPOINT			UINT32_C(32954)	/* region-xform tack point */
#define TIFFTAG_REGIONWARPCORNERS		UINT32_C(32955)	/* warp quadrilateral */
#define TIFFTAG_REGIONAFFINE			UINT32_C(32956)	/* affine transformation mat */
/* tags 32995-32999 are private tags registered to SGI */
#define	TIFFTAG_MATTEING				UINT32_C(32995)	/* SHORT $use ExtraSamples */
#define	TIFFTAG_DATATYPE				UINT32_C(32996)	/* SHORT $use SampleFormat */
#define	TIFFTAG_IMAGEDEPTH				UINT32_C(32997)	/* LONG z depth of image */
#define	TIFFTAG_TILEDEPTH				UINT32_C(32998)	/* LONG z depth/data tile */
/* tags 33300-33309 are private tags registered to Pixar */
/*
 * TIFFTAG_PIXAR_IMAGEFULLWIDTH and TIFFTAG_PIXAR_IMAGEFULLLENGTH
 * are set when an image has been cropped out of a larger image.  
 * They reflect the size of the original uncropped image.
 * The TIFFTAG_XPOSITION and TIFFTAG_YPOSITION can be used
 * to determine the position of the smaller image in the larger one.
 */
#define TIFFTAG_PIXAR_IMAGEFULLWIDTH    UINT32_C(33300)   /* LONG full image size in x */
#define TIFFTAG_PIXAR_IMAGEFULLLENGTH   UINT32_C(33301)   /* LONG full image size in y */
 /* Tags 33302-33306 are used to identify special image modes and data
  * used by Pixar's texture formats.
  */
#define TIFFTAG_PIXAR_TEXTUREFORMAT		UINT32_C(33302)	/* ASCII texture map format */
#define TIFFTAG_PIXAR_WRAPMODES			UINT32_C(33303)	/* ASCII s & t wrap modes */
#define TIFFTAG_PIXAR_FOVCOT			UINT32_C(33304)	/* FLOAT cotan(fov) for env. maps */
#define TIFFTAG_PIXAR_MATRIX_WORLDTOSCREEN UINT32_C(33305) /* FLOAT */
#define TIFFTAG_PIXAR_MATRIX_WORLDTOCAMERA UINT32_C(33306) /* FLOAT */
/* tag 33405 is a private tag registered to Eastman Kodak */
#define TIFFTAG_WRITERSERIALNUMBER  	UINT32_C(33405) /* device serial number */
#define TIFFTAG_CFAREPEATPATTERNDIM		UINT32_C(33421)	/* SHORT dimensions of CFA pattern */
#define TIFFTAG_CFAPATTERN				UINT32_C(33422)	/* BYTE color filter array pattern */
/* tag 33432 is listed in the 6.0 spec w/ unknown ownership */
#define	TIFFTAG_COPYRIGHT				UINT32_C(33432)	/* ASCII copyright string */
/* IPTC TAG from RichTIFF specifications */
#define TIFFTAG_RICHTIFFIPTC			UINT32_C(33723) /* LONG */
/* 34016-34029 are reserved for ANSI IT8 TIFF/IT <dkelly@apago.com) */
#define TIFFTAG_IT8SITE					UINT32_C(34016)	/* site name */
#define TIFFTAG_IT8COLORSEQUENCE		UINT32_C(34017)	/* color seq. [RGB,CMYK,etc] */
#define TIFFTAG_IT8HEADER				UINT32_C(34018)	/* DDES Header */
#define TIFFTAG_IT8RASTERPADDING		UINT32_C(34019)	/* raster scanline padding */
#define TIFFTAG_IT8BITSPERRUNLENGTH		UINT32_C(34020)	/* # of bits in short run */
#define TIFFTAG_IT8BITSPEREXTENDEDRUNLENGTH UINT32_C(34021) /* # of bits in long run */
#define TIFFTAG_IT8COLORTABLE			UINT32_C(34022)	/* LW colortable */
#define TIFFTAG_IT8IMAGECOLORINDICATOR	UINT32_C(34023)	/* BP/BL image color switch */
#define TIFFTAG_IT8BKGCOLORINDICATOR	UINT32_C(34024)	/* BP/BL bg color switch */
#define TIFFTAG_IT8IMAGECOLORVALUE		UINT32_C(34025)	/* BP/BL image color value */
#define TIFFTAG_IT8BKGCOLORVALUE		UINT32_C(34026)	/* BP/BL bg color value */
#define TIFFTAG_IT8PIXELINTENSITYRANGE	UINT32_C(34027)	/* MP pixel intensity value */
#define TIFFTAG_IT8TRANSPARENCYINDICATOR UINT32_C(34028)	/* HC transparency switch */
#define TIFFTAG_IT8COLORCHARACTERIZATION UINT32_C(34029)	/* color character. table */
#define TIFFTAG_IT8HCUSAGE				UINT32_C(34030)	/* HC usage indicator */
#define TIFFTAG_IT8TRAPINDICATOR		UINT32_C(34031)	/* Trapping indicator (untrapped=0, trapped=1) */
#define TIFFTAG_IT8CMYKEQUIVALENT		UINT32_C(34032)	/* CMYK color equivalents */
/* tags 34232-34236 are private tags registered to Texas Instruments */
#define TIFFTAG_FRAMECOUNT              UINT32_C(34232)   /* Sequence Frame Count */
/* tag 34377 is private tag registered to Adobe for PhotoShop */
#define TIFFTAG_PHOTOSHOP				UINT32_C(34377) /* BYTE */
/* tags 34665, 34853 and 40965 are documented in EXIF specification */
#define TIFFTAG_EXIFIFD					UINT32_C(34665)	/* IFD8 Pointer to EXIF private directory */
/* tag 34750 is a private tag registered to Adobe? */
#define TIFFTAG_ICCPROFILE				UINT32_C(34675)	/* UNDEFINED ICC profile data */
#define TIFFTAG_IMAGELAYER				UINT32_C(34732)	/* LONG !TIFF/FX image layer information */
/* tag 34750 is a private tag registered to Pixel Magic */
#define	TIFFTAG_JBIGOPTIONS				UINT32_C(34750)	/* JBIG options */
#define TIFFTAG_GPSIFD					UINT32_C(34853)	/* IFD8 Pointer to GPS private directory */
/* tags 34908-34914 are private tags registered to SGI */
#define	TIFFTAG_FAXRECVPARAMS			UINT32_C(34908)	/* LONG encoded Class 2 ses. parms */
#define	TIFFTAG_FAXSUBADDRESS			UINT32_C(34909)	/* ASCII received SubAddr string */
#define	TIFFTAG_FAXRECVTIME				UINT32_C(34910)	/* LONG receive time (secs) */
#define	TIFFTAG_FAXDCS					UINT32_C(34911)	/* ASCII encoded fax ses. params, Table 2/T.30 */
/* tags 37439-37443 are registered to SGI <gregl@sgi.com> */
#define TIFFTAG_STONITS					UINT32_C(37439)	/* DOUBLE Sample value to Nits */
/* tag 34929 is a private tag registered to FedEx */
#define	TIFFTAG_FEDEX_EDR				UINT32_C(34929)	/* unknown use */
#define TIFFTAG_INTEROPERABILITYIFD		UINT32_C(40965)	/* IFD8 Pointer to Interoperability private directory */
/* tags 50674 to 50677 are reserved for ESRI */
#define TIFFTAG_LERC_PARAMETERS     	UINT32_C(50674) /* Stores LERC version and additional compression method */
/* Adobe Digital Negative (DNG) format tags */
#define TIFFTAG_DNGVERSION				UINT32_C(50706)	/* BYTE &DNG version number */
#define TIFFTAG_DNGBACKWARDVERSION		UINT32_C(50707)	/* BYTE &DNG compatibility version */
#define TIFFTAG_UNIQUECAMERAMODEL		UINT32_C(50708)	/* ASCII &name for the camera model */
#define TIFFTAG_LOCALIZEDCAMERAMODEL	UINT32_C(50709)	/* BYTE &localized camera model name */
#define TIFFTAG_CFAPLANECOLOR			UINT32_C(50710)	/* BYTE &CFAPattern->LinearRaw space mapping */
#define TIFFTAG_CFALAYOUT				UINT32_C(50711)	/* SHORT &spatial layout of the CFA */
#define TIFFTAG_LINEARIZATIONTABLE		UINT32_C(50712)	/* SHORT &lookup table description */
#define TIFFTAG_BLACKLEVELREPEATDIM		UINT32_C(50713)	/* SHORT &repeat pattern size for the BlackLevel tag */
#define TIFFTAG_BLACKLEVEL				UINT32_C(50714)	/* RATIONAL &zero light encoding level */
#define TIFFTAG_BLACKLEVELDELTAH		UINT32_C(50715)	/* SRATIONAL &zero light encoding level differences (columns) */
#define TIFFTAG_BLACKLEVELDELTAV		UINT32_C(50716)	/* SRATIONAL &zero light encoding leveldifferences (rows) */
#define TIFFTAG_WHITELEVEL				UINT32_C(50717)	/* LONG &fully saturated encoding level */
#define TIFFTAG_DEFAULTSCALE			UINT32_C(50718)	/* RATIONAL &default scale factors */
#define TIFFTAG_DEFAULTCROPORIGIN		UINT32_C(50719)	/* RATIONAL &origin of the final image area */
#define TIFFTAG_DEFAULTCROPSIZE			UINT32_C(50720)	/* RATIONAL &size of the final image area */
#define TIFFTAG_COLORMATRIX1			UINT32_C(50721)	/* SRATIONAL &XYZ->reference color space transformation matrix 1 */
#define TIFFTAG_COLORMATRIX2			UINT32_C(50722)	/* SRATIONAL &XYZ->reference color space transformation matrix 2 */
#define TIFFTAG_CAMERACALIBRATION1		UINT32_C(50723)	/* SRATIONAL &calibration matrix 1 */
#define TIFFTAG_CAMERACALIBRATION2		UINT32_C(50724)	/* SRATIONAL &calibration matrix 2 */
#define TIFFTAG_REDUCTIONMATRIX1		UINT32_C(50725)	/* SRATIONAL &dimensionality reduction matrix 1 */
#define TIFFTAG_REDUCTIONMATRIX2		UINT32_C(50726)	/* SRATIONAL &dimensionality reduction matrix 2 */
#define TIFFTAG_ANALOGBALANCE			UINT32_C(50727)	/* RATIONAL &gain applied the stored raw values */
#define TIFFTAG_ASSHOTNEUTRAL			UINT32_C(50728)	/* RATIONAL &selected white balance in linear reference space */
#define TIFFTAG_ASSHOTWHITEXY			UINT32_C(50729)	/* RATIONAL &selected white balance in x-y chromaticity coordinates */
#define TIFFTAG_BASELINEEXPOSURE		UINT32_C(50730)	/* SRATIONAL &how much to move the zero point */
#define TIFFTAG_BASELINENOISE			UINT32_C(50731)	/* RATIONAL &relative noise level */
#define TIFFTAG_BASELINESHARPNESS		UINT32_C(50732)	/* RATIONAL &relative amount of sharpening */
#define TIFFTAG_BAYERGREENSPLIT			UINT32_C(50733)	/* LONG &how closely the values of
														   the green pixels in the
														   blue/green rows track the
														   values of the green pixels
														   in the red/green rows */
#define TIFFTAG_LINEARRESPONSELIMIT		UINT32_C(50734)	/* RATIONAL &non-linear encoding range */
#define TIFFTAG_CAMERASERIALNUMBER		UINT32_C(50735)	/* ASCII &camera's serial number */
#define TIFFTAG_LENSINFO				UINT32_C(50736)	/* RATIONAL info about the lens */
#define TIFFTAG_CHROMABLURRADIUS		UINT32_C(50737)	/* RATIONAL &chroma blur radius */
#define TIFFTAG_ANTIALIASSTRENGTH		UINT32_C(50738)	/* RATIONAL &relative strength of the camera's anti-alias filter */
#define TIFFTAG_SHADOWSCALE				UINT32_C(50739)	/* RATIONAL &used by Adobe Camera Raw */
#define TIFFTAG_DNGPRIVATEDATA			UINT32_C(50740)	/* BYTE &manufacturer's private data */
#define TIFFTAG_MAKERNOTESAFETY			UINT32_C(50741)	/* SHORT &whether the EXIF MakerNote
														   tag is safe to preserve
														   along with the rest of the
														   EXIF data */
#define	TIFFTAG_CALIBRATIONILLUMINANT1	UINT32_C(50778)	/* SHORT &illuminant 1 */
#define TIFFTAG_CALIBRATIONILLUMINANT2	UINT32_C(50779)	/* SHORT &illuminant 2 */
#define TIFFTAG_BESTQUALITYSCALE		UINT32_C(50780)	/* RATIONAL &best quality multiplier */
#define TIFFTAG_RAWDATAUNIQUEID			UINT32_C(50781)	/* BYTE &unique identifier for the raw image data */
#define TIFFTAG_ORIGINALRAWFILENAME		UINT32_C(50827)	/* BYTE &file name of the original raw file */
#define TIFFTAG_ORIGINALRAWFILEDATA		UINT32_C(50828)	/* BYTE &contents of the original raw file */
#define TIFFTAG_ACTIVEAREA				UINT32_C(50829)	/* LONG &active (non-masked) pixels of the sensor */
#define TIFFTAG_MASKEDAREAS				UINT32_C(50830)	/* LONG &list of coordinates of fully masked pixels */
#define TIFFTAG_ASSHOTICCPROFILE		UINT32_C(50831)	/* UNDEFINED &these two tags used to */
#define TIFFTAG_ASSHOTPREPROFILEMATRIX	UINT32_C(50832)	/* SRATIONAL map cameras's color space into ICC profile space */
#define TIFFTAG_CURRENTICCPROFILE		UINT32_C(50833)	/* UNDEFINED & */
#define TIFFTAG_CURRENTPREPROFILEMATRIX	UINT32_C(50834)	/* SRATIONAL & */
/* tag 65535 is an undefined tag used by Eastman Kodak */
#define TIFFTAG_DCSHUESHIFTVALUES       UINT32_C(65535)   /* hue shift correction data */

/*
 * The following are ``pseudo tags'' that can be used to control
 * codec-specific functionality.  These tags are not written to file.
 * Note that these values start at 0xffff+1 so that they'll never
 * collide with Aldus-assigned tags.
 *
 * If you want your private pseudo tags ``registered'' (i.e. added to
 * this file), please post a bug report via the tracking system at
 * http://www.remotesensing.org/libtiff/bugs.html with the appropriate
 * C definitions to add.
 */
#define	TIFFTAG_FAXMODE					UINT32_C(65536)	/* INT Group 3/4 format control */
#define	    FAXMODE_CLASSIC					0x0000		/* default, include RTC */
#define	    FAXMODE_NORTC					0x0001		/* no RTC at end of data */
#define	    FAXMODE_NOEOL					0x0002		/* no EOL code at end of row */
#define	    FAXMODE_BYTEALIGN				0x0004		/* byte align row */
#define	    FAXMODE_WORDALIGN				0x0008		/* word align row */
#define	    FAXMODE_CLASSF					FAXMODE_NORTC	/* TIFF Class F */
#define	TIFFTAG_JPEGQUALITY				UINT32_C(65537)	/* INT Compression quality level */
/* Note: quality level is on the IJG 0-100 scale.  Default value is 75 */
#define	TIFFTAG_JPEGCOLORMODE			UINT32_C(65538)	/* INT Auto RGB<=>YCbCr convert? */
#define	    JPEGCOLORMODE_RAW				0x0000		/* no conversion (default) */
#define	    JPEGCOLORMODE_RGB				0x0001		/* do auto conversion */
#define	TIFFTAG_JPEGTABLESMODE			UINT32_C(65539)	/* INT What to put in JPEGTables */
#define	    JPEGTABLESMODE_QUANT 			0x0001		/* include quantization tbls */
#define	    JPEGTABLESMODE_HUFF				0x0002		/* include Huffman tbls */
/* Note: default is JPEGTABLESMODE_QUANT | JPEGTABLESMODE_HUFF */
#define	TIFFTAG_FAXFILLFUNC				UINT32_C(65540)	/* G3/G4 fill function */
#define	TIFFTAG_PIXARLOGDATAFMT			UINT32_C(65549)	/* INT PixarLogCodec I/O data sz */
#define	    PIXARLOGDATAFMT_8BIT			0	/* regular u_char samples */
#define	    PIXARLOGDATAFMT_8BITABGR		1	/* ABGR-order u_chars */
#define	    PIXARLOGDATAFMT_11BITLOG		2	/* 11-bit log-encoded (raw) */
#define	    PIXARLOGDATAFMT_12BITPICIO		3	/* as per PICIO (1.0==2048) */
#define	    PIXARLOGDATAFMT_16BIT			4	/* signed short samples */
#define	    PIXARLOGDATAFMT_FLOAT			5	/* IEEE float samples */
/* 65550-65556 are allocated to Oceana Matrix <dev@oceana.com> */
#define TIFFTAG_DCSIMAGERTYPE       	UINT32_C(65550)   /* imager model & filter */
#define     DCSIMAGERMODEL_M3       	    0       /* M3 chip (1280 x 1024) */
#define     DCSIMAGERMODEL_M5       	    1       /* M5 chip (1536 x 1024) */
#define     DCSIMAGERMODEL_M6       	    2       /* M6 chip (3072 x 2048) */
#define     DCSIMAGERFILTER_IR      	    0       /* infrared filter */
#define     DCSIMAGERFILTER_MONO    	    1       /* monochrome filter */
#define     DCSIMAGERFILTER_CFA     	    2       /* color filter array */
#define     DCSIMAGERFILTER_OTHER   	    3       /* other filter */
#define TIFFTAG_DCSINTERPMODE       	UINT32_C(65551)   /* interpolation mode */
#define     DCSINTERPMODE_NORMAL    	    0x0     /* whole image, default */
#define     DCSINTERPMODE_PREVIEW   	    0x1     /* preview of image (384x256) */
#define TIFFTAG_DCSBALANCEARRAY     	UINT32_C(65552)   /* color balance values */
#define TIFFTAG_DCSCORRECTMATRIX    	UINT32_C(65553)   /* color correction values */
#define TIFFTAG_DCSGAMMA            	UINT32_C(65554)   /* gamma value */
#define TIFFTAG_DCSTOESHOULDERPTS   	UINT32_C(65555)   /* toe & shoulder points */
#define TIFFTAG_DCSCALIBRATIONFD    	UINT32_C(65556)   /* calibration file desc */
/* Note: quality level is on the ZLIB 1-9 scale. Default value is -1 */
#define	TIFFTAG_ZIPQUALITY				UINT32_C(65557)	/* INT compression quality level */
#define	TIFFTAG_PIXARLOGQUALITY			UINT32_C(65558)	/* INT PixarLog uses same scale */
/* 65559 is allocated to Oceana Matrix <dev@oceana.com> */
#define TIFFTAG_DCSCLIPRECTANGLE		UINT32_C(65559)	/* area of image to acquire */
#define TIFFTAG_SGILOGDATAFMT			UINT32_C(65560)	/* INT SGILog user data format */
#define     SGILOGDATAFMT_FLOAT				0	/* IEEE float samples */
#define     SGILOGDATAFMT_16BIT				1	/* 16-bit samples */
#define     SGILOGDATAFMT_RAW				2	/* uninterpreted data */
#define     SGILOGDATAFMT_8BIT				3	/* 8-bit RGB monitor values */
#define TIFFTAG_SGILOGENCODE			UINT32_C(65561) /* INT SGILog data encoding control*/
#define     SGILOGENCODE_NODITHER			0     /* do not dither encoded values*/
#define     SGILOGENCODE_RANDITHER			1     /* randomly dither encd values */
#define	TIFFTAG_LZMAPRESET				UINT32_C(65562)	/* INT LZMA2 preset (compression level) */
#define TIFFTAG_PERSAMPLE       		UINT32_C(65563)	/* interface for per sample tags */
#define     PERSAMPLE_MERGED        		0	/* present as a single value */
#define     PERSAMPLE_MULTI         		1	/* present as multiple values */
#define TIFFTAG_ZSTD_LEVEL      		UINT32_C(65564)    /* ZSTD compression level */
#define TIFFTAG_LERC_VERSION        	UINT32_C(65565) /* LERC version */
#define     LERC_VERSION_2_4        	    4
#define TIFFTAG_LERC_ADD_COMPRESSION    UINT32_C(65566) /* LERC additional compression */
#define     LERC_ADD_COMPRESSION_NONE   	0
#define     LERC_ADD_COMPRESSION_DEFLATE	1
#define     LERC_ADD_COMPRESSION_ZSTD   	2
#define TIFFTAG_LERC_MAXZERROR      	UINT32_C(65567)    /* LERC maximum error */
#define TIFFTAG_WEBP_LEVEL		  		UINT32_C(65568)	/* WebP compression level: WARNING not registered in Adobe-maintained registry */
#define TIFFTAG_WEBP_LOSSLESS			UINT32_C(65569)	/* WebP lossless/lossy : WARNING not registered in Adobe-maintained registry */

/*
 * EXIF tags
 */
#define EXIFTAG_EXPOSURETIME			UINT32_C(33434)	/* Exposure time */
#define EXIFTAG_FNUMBER					UINT32_C(33437)	/* F number */
#define EXIFTAG_EXPOSUREPROGRAM			UINT32_C(34850)	/* Exposure program */
#define EXIFTAG_SPECTRALSENSITIVITY		UINT32_C(34852)	/* Spectral sensitivity */
#define EXIFTAG_ISOSPEEDRATINGS			UINT32_C(34855)	/* ISO speed rating */
#define EXIFTAG_OECF					UINT32_C(34856)	/* Optoelectric conversion factor */
#define EXIFTAG_EXIFVERSION				UINT32_C(36864)	/* Exif version */
#define EXIFTAG_DATETIMEORIGINAL		UINT32_C(36867)	/* Date and time of original data generation */
#define EXIFTAG_DATETIMEDIGITIZED		UINT32_C(36868)	/* Date and time of digital data generation */
#define EXIFTAG_COMPONENTSCONFIGURATION	UINT32_C(37121)	/* Meaning of each component */
#define EXIFTAG_COMPRESSEDBITSPERPIXEL	UINT32_C(37122)	/* Image compression mode */
#define EXIFTAG_SHUTTERSPEEDVALUE		UINT32_C(37377)	/* Shutter speed */
#define EXIFTAG_APERTUREVALUE			UINT32_C(37378)	/* Aperture */
#define EXIFTAG_BRIGHTNESSVALUE			UINT32_C(37379)	/* Brightness */
#define EXIFTAG_EXPOSUREBIASVALUE		UINT32_C(37380)	/* Exposure bias */
#define EXIFTAG_MAXAPERTUREVALUE		UINT32_C(37381)	/* Maximum lens aperture */
#define EXIFTAG_SUBJECTDISTANCE			UINT32_C(37382)	/* Subject distance */
#define EXIFTAG_METERINGMODE			UINT32_C(37383)	/* Metering mode */
#define EXIFTAG_LIGHTSOURCE				UINT32_C(37384)	/* Light source */
#define EXIFTAG_FLASH					UINT32_C(37385)	/* Flash */
#define EXIFTAG_FOCALLENGTH				UINT32_C(37386)	/* Lens focal length */
#define EXIFTAG_SUBJECTAREA				UINT32_C(37396)	/* Subject area */
#define EXIFTAG_MAKERNOTE				UINT32_C(37500)	/* Manufacturer notes */
#define EXIFTAG_USERCOMMENT				UINT32_C(37510)	/* User comments */
#define EXIFTAG_SUBSECTIME				UINT32_C(37520)	/* DateTime subseconds */
#define EXIFTAG_SUBSECTIMEORIGINAL		UINT32_C(37521)	/* DateTimeOriginal subseconds */
#define EXIFTAG_SUBSECTIMEDIGITIZED		UINT32_C(37522)	/* DateTimeDigitized subseconds */
#define EXIFTAG_FLASHPIXVERSION			UINT32_C(40960)	/* Supported Flashpix version */
#define EXIFTAG_COLORSPACE				UINT32_C(40961)	/* Color space information */
#define EXIFTAG_PIXELXDIMENSION			UINT32_C(40962)	/* Valid image width */
#define EXIFTAG_PIXELYDIMENSION			UINT32_C(40963)	/* Valid image height */
#define EXIFTAG_RELATEDSOUNDFILE		UINT32_C(40964)	/* Related audio file */
#define EXIFTAG_FLASHENERGY				UINT32_C(41483)	/* Flash energy */
#define EXIFTAG_SPATIALFREQUENCYRESPONSE UINT32_C(41484)	/* Spatial frequency response */
#define EXIFTAG_FOCALPLANEXRESOLUTION	 UINT32_C(41486)	/* Focal plane X resolution */
#define EXIFTAG_FOCALPLANEYRESOLUTION	 UINT32_C(41487)	/* Focal plane Y resolution */
#define EXIFTAG_FOCALPLANERESOLUTIONUNIT UINT32_C(41488)	/* Focal plane resolution unit */
#define EXIFTAG_SUBJECTLOCATION			UINT32_C(41492)	/* Subject location */
#define EXIFTAG_EXPOSUREINDEX			UINT32_C(41493)	/* Exposure index */
#define EXIFTAG_SENSINGMETHOD			UINT32_C(41495)	/* Sensing method */
#define EXIFTAG_FILESOURCE				UINT32_C(41728)	/* File source */
#define EXIFTAG_SCENETYPE				UINT32_C(41729)	/* Scene type */
#define EXIFTAG_CFAPATTERN				UINT32_C(41730)	/* CFA pattern */
#define EXIFTAG_CUSTOMRENDERED			UINT32_C(41985)	/* Custom image processing */
#define EXIFTAG_EXPOSUREMODE			UINT32_C(41986)	/* Exposure mode */
#define EXIFTAG_WHITEBALANCE			UINT32_C(41987)	/* White balance */
#define EXIFTAG_DIGITALZOOMRATIO		UINT32_C(41988)	/* Digital zoom ratio */
#define EXIFTAG_FOCALLENGTHIN35MMFILM	UINT32_C(41989)	/* Focal length in 35 mm film */
#define EXIFTAG_SCENECAPTURETYPE		UINT32_C(41990)	/* Scene capture type */
#define EXIFTAG_GAINCONTROL				UINT32_C(41991)	/* Gain control */
#define EXIFTAG_CONTRAST				UINT32_C(41992)	/* Contrast */
#define EXIFTAG_SATURATION				UINT32_C(41993)	/* Saturation */
#define EXIFTAG_SHARPNESS				UINT32_C(41994)	/* Sharpness */
#define EXIFTAG_DEVICESETTINGDESCRIPTION UINT32_C(41995)	/* Device settings description */
#define EXIFTAG_SUBJECTDISTANCERANGE	UINT32_C(41996)	/* Subject distance range */
#define EXIFTAG_GAINCONTROL				UINT32_C(41991)	/* Gain control */
#define EXIFTAG_GAINCONTROL				UINT32_C(41991)	/* Gain control */
#define EXIFTAG_IMAGEUNIQUEID			UINT32_C(42016)	/* Unique image ID */

#endif /* _TIFF_ */
